VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "MouseIO"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'==================================================================
'
' Mouse Control
'
' 30 September 2001
'
' You're free in using this software, as long as the
' orignal copyright/author information remains intact.
' In other words, respecting the person(s) who made it
' possible for you to view/learn something of this software.
'
' Copyright  2001 - Diederik van der Boor
' http://www.codingdomain.com
'
'==================================================================


'------------------------------------------------------------------
' API's

Private Type POINTAPI
  X As Long
  Y As Long
End Type

Private Type RECT
  Left As Long
  Top As Long
  Right As Long
  Bottom As Long
End Type

Private Type CURSORINFO
  cbSize As Long
  flags As Long ' Visible
  hCursor As Long
  ptScreenPos As POINTAPI
End Type

Private Enum GetClassLong
  GCL_CBCLSEXTRA = (-20)
  GCL_CBWNDEXTRA = (-18)
  GCL_CONVERSION = &H1
  GCL_HBRBACKGROUND = (-10)
  GCL_HCURSOR = (-12)
  GCL_HICON = (-14)
  GCL_HMODULE = (-16)
  GCL_MENUNAME = (-8)
  GCL_REVERSE_LENGTH = &H3
  GCL_REVERSECONVERSION = &H2
  GCL_STYLE = (-26)
  GCL_WNDPROC = (-24)
End Enum

Private Declare Function GetClassLong Lib "user32" Alias "GetClassLongA" (ByVal hWnd As Long, ByVal nIndex As GetClassLong) As Long
Private Declare Function SetClassLong Lib "user32" Alias "SetClassLongA" (ByVal hWnd As Long, ByVal nIndex As GetClassLong, ByVal dwNewLong As Long) As Long
Private Declare Function IsWindow Lib "user32" (ByVal hWnd As Long) As Long

'Private Declare Function GetCursor Lib "user32" () As Long
'Private Declare Function SetCursor Lib "user32" (ByVal hCursor As Long) As Long
Private Declare Function GetCursorPos Lib "user32" (lpPoint As POINTAPI) As Long
Private Declare Function SetCursorPos Lib "user32" (ByVal X As Long, ByVal Y As Long) As Long
Private Declare Function ShowCursor Lib "user32" (ByVal bShow As Long) As Long
Private Declare Function GetCursorInfo Lib "user32" (pci As CURSORINFO) As Long

Private Declare Function GetClipCursor Lib "user32" (lprc As RECT) As Long
Private Declare Function ClipCursor Lib "user32" (lpRect As Any) As Long



'------------------------------------------------------------------
' Mouse Position

Public Property Get X() As Long
  X = Mouse_Pos().X
End Property

Public Property Get Y() As Long
  Y = Mouse_Pos().Y
End Property

Public Sub Move(X As Long, Y As Long)
  If SetCursorPos(X, Y) = 0 Then _
    Err.Raise 12345, "MouseIO", "Could not position mouse!"
End Sub

Public Property Let X(NewValue As Long)
  Call Me.Move(NewValue, Mouse_Pos().Y)
End Property

Public Property Let Y(NewValue As Long)
  Call Me.Move(Mouse_Pos().Y, NewValue)
End Property


'------------------------------------------------------------------
' Cursor (under construction)

'Public Property Get Cursor(hWnd As Long) As Cursor
'  If IsWindow(hWnd) = 0 Then Error 5
'  Dim CurCursor As New Cursor
'  CurCursor.hCursor = GetClassLong(hWnd, GCL_HCURSOR)
'  Set Cursor = CurCursor
'End Property
'
'Public Property Let Cursor(hWnd As Long, NewValue As Cursor)
'  If IsWindow(hWnd) = 0 Then Error 5
'  Dim PrevCursor As Long
'  PrevCursor = SetClassLong(hWnd, GCL_HCURSOR, NewValue.hCursor)
'End Property


'------------------------------------------------------------------
' Clip the mouse

Public Sub Clip(Left As Long, Top As Long, Right As Long, Bottom As Long)
  Dim rctClip As RECT
  
  rctClip.Left = Left
  rctClip.Top = Top
  rctClip.Right = Right
  rctClip.Bottom = Bottom
  If ClipCursor(rctClip) = 0 Then _
    Err.Raise 12345, "MouseIO", "Could not clip cursor!"
End Sub

Public Sub UnClip()
  If ClipCursor(ByVal 0&) = 0 Then _
    Err.Raise 12345, "MouseIO", "Could not unclip cursor!"
End Sub


'------------------------------------------------------------------
' Other Clip properties

Public Property Get ClipLeft() As Long
  ClipLeft = Mouse_ClipArea().Left
End Property

Public Property Let ClipLeft(NewValue As Long)
  With Mouse_ClipArea()
    Call Me.Clip(NewValue, .Top, .Right, .Bottom)
  End With
End Property

Public Property Get ClipTop() As Long
  ClipTop = Mouse_ClipArea().Top
End Property

Public Property Let ClipTop(NewValue As Long)
  With Mouse_ClipArea()
    Call Me.Clip(.Left, NewValue, .Right, .Bottom)
  End With
End Property

Public Property Get ClipRight() As Long
  ClipRight = Mouse_ClipArea().Right
End Property

Public Property Let ClipRight(NewValue As Long)
  With Mouse_ClipArea()
    Call Me.Clip(.Left, .Top, NewValue, .Bottom)
  End With
End Property

Public Property Get ClipBottom() As Long
  ClipBottom = Mouse_ClipArea().Bottom
End Property

Public Property Let ClipBottom(NewValue As Long)
  With Mouse_ClipArea()
    Call Me.Clip(.Left, .Top, .Right, NewValue)
  End With
End Property


'------------------------------------------------------------------
' Mouse Visible

Public Function ShowAdd() As Long
  ShowAdd = ShowCursor(1)
End Function

Public Function HideAdd() As Long
  HideAdd = ShowCursor(0)
End Function

Public Property Get Visible() As Boolean
  Dim CurInfo As CURSORINFO
  If GetCursorInfo(CurInfo) = 0 Then Err.Raise 12345, "Mouse", "Could not get mouse info!"
  Visible = CurInfo.flags
End Property

Public Property Let Visible(NewValue As Boolean)
  Dim Counter As Long
  
  ' Get current (add returns previous) counter value, restore value
  Counter = Me.HideAdd
  Call Me.ShowAdd ' Return value should be Counter+1
  
  If NewValue = True Then
    Do Until Counter > 0
      Counter = (Me.ShowAdd() + 1)
    Loop
  Else
    Do Until Counter < 1
      Counter = (Me.HideAdd() - 1)
    Loop
  End If
End Property


'------------------------------------------------------------------
' Helpers

Private Function Mouse_FileExist(FileName As String) As Boolean
  Mouse_FileExist = Len(Dir(FileName)) And Len(FileName) <> 0
End Function

Private Function Mouse_Pos() As POINTAPI
  Dim Pos As POINTAPI
  If GetCursorPos(Pos) = 0 Then Err.Raise 12345, "MouseIO", "Could not get mouse position!"
  Mouse_Pos = Pos
End Function

Private Function Mouse_ClipArea() As RECT
  If GetClipCursor(Mouse_ClipArea) = 0 Then Err.Raise 12345, "Mouse", "Could not get clipping area!"
End Function
